@@ -63,8 +63,9 @@ static void       markup_dir_free                  (MarkupDir  *dir);
 static gboolean   markup_dir_needs_sync            (MarkupDir  *dir);
 static gboolean   markup_dir_sync                  (MarkupDir  *dir);
 static char*      markup_dir_build_path            (MarkupDir  *dir,
-						    gboolean    with_data_file,
-						    gboolean    subtree_data_file);
+                                                    gboolean    filesystem_path,
+                                                    gboolean    with_data_file,
+                                                    gboolean    subtree_data_file);
 static void       markup_dir_set_entries_need_save (MarkupDir  *dir);
 
 static MarkupEntry* markup_entry_new  (MarkupDir   *dir,
@@ -258,6 +259,20 @@ markup_dir_queue_sync (MarkupDir *dir)
     }
 }
 
+static inline char *
+markup_dir_build_file_path (MarkupDir *dir,
+                            gboolean   subtree_data_file)
+{
+  return markup_dir_build_path (dir, TRUE, TRUE, subtree_data_file);
+}
+
+static inline char *
+markup_dir_build_dir_path (MarkupDir *dir,
+                           gboolean   filesystem_path)
+{
+  return markup_dir_build_path (dir, filesystem_path, FALSE, FALSE);
+}
+
 static MarkupDir*
 markup_tree_get_dir_internal (MarkupTree *tree,
                               const char *full_key,
@@ -361,7 +376,7 @@ load_subtree (MarkupDir *dir)
   GError *tmp_err = NULL;
   char *markup_file;
 
-  markup_file = markup_dir_build_path (dir, TRUE, TRUE);
+  markup_file = markup_dir_build_file_path (dir, TRUE);
   if (!gconf_file_exists (markup_file))
     {
       g_free (markup_file);
@@ -427,7 +442,7 @@ load_entries (MarkupDir *dir)
 	  /* this message is debug-only because it usually happens
 	   * when creating a new directory
 	   */
-	  markup_file = markup_dir_build_path (dir, TRUE, FALSE);
+	  markup_file = markup_dir_build_file_path (dir, FALSE);
 	  gconf_log (GCL_DEBUG,
 		     "Failed to load file \"%s\": %s",
 		     markup_file, tmp_err->message);
@@ -467,7 +482,7 @@ load_subdirs (MarkupDir *dir)
   if (load_subtree (dir))
     return TRUE;
 
-  markup_dir = markup_dir_build_path (dir, FALSE, FALSE);
+  markup_dir = markup_dir_build_dir_path (dir, TRUE);
   
   dp = opendir (markup_dir);
   
@@ -814,8 +829,8 @@ delete_useless_subdirs (MarkupDir *dir)
 	      char *fs_dirname;
 	      char *fs_filename;
           
-	      fs_dirname = markup_dir_build_path (subdir, FALSE, FALSE);
-	      fs_filename = markup_dir_build_path (subdir, TRUE, subdir->save_as_subtree);
+	      fs_dirname = markup_dir_build_dir_path (subdir, TRUE);
+	      fs_filename = markup_dir_build_file_path (subdir, subdir->save_as_subtree);
 
 	      if (unlink (fs_filename) < 0)
 		{
@@ -961,9 +976,9 @@ markup_dir_sync (MarkupDir *dir)
   /* Sanitize the entries */
   clean_old_local_schemas_recurse (dir, dir->save_as_subtree);
   
-  fs_dirname = markup_dir_build_path (dir, FALSE, FALSE);
-  fs_filename = markup_dir_build_path (dir, TRUE, FALSE);
-  fs_subtree = markup_dir_build_path (dir, TRUE, TRUE);
+  fs_dirname = markup_dir_build_dir_path (dir, TRUE);
+  fs_filename = markup_dir_build_file_path (dir, FALSE);
+  fs_subtree = markup_dir_build_file_path (dir, TRUE);
 
   /* For a dir to be loaded as a subdir, it must have a
    * %gconf.xml file, even if it has no entries in that
@@ -1101,6 +1116,7 @@ markup_dir_sync (MarkupDir *dir)
 
 static char*
 markup_dir_build_path (MarkupDir  *dir,
+                       gboolean    filesystem_path,
                        gboolean    with_data_file,
 		       gboolean    subtree_data_file)
 {
@@ -1109,6 +1125,8 @@ markup_dir_build_path (MarkupDir  *dir,
   GSList *tmp;
   MarkupDir *iter;
 
+  g_assert (filesystem_path || !with_data_file);
+
   components = NULL;
   iter = dir;
   while (iter->parent != NULL) /* exclude root dir */
@@ -1117,7 +1135,11 @@ markup_dir_build_path (MarkupDir  *dir,
       iter = iter->parent;
     }
 
-  name = g_string_new (dir->tree->dirname);
+  if (filesystem_path)
+    name = g_string_new (dir->tree->dirname);
+  else
+    name = g_string_new (components ? NULL : "/");
+
   tmp = components;
   while (tmp != NULL)
     {
@@ -2213,7 +2235,7 @@ parse_value_element (GMarkupParseContext  *context,
               case GCONF_VALUE_PAIR:
               case GCONF_VALUE_SCHEMA:
                 set_error (error, context, GCONF_ERROR_PARSE_ERROR,
-                           _("Invalid car_type \"%s\" on <%s>"),
+                           _("Invalid first-element type \"%s\" on <%s>"),
                            car_type, element_name);
                 return;
                 break;
@@ -3120,7 +3142,7 @@ parse_tree (MarkupDir  *root,
   char *text;
   gsize length;
 
-  filename = markup_dir_build_path (root, TRUE, parse_subtree);
+  filename = markup_dir_build_file_path (root, parse_subtree);
   
   parse_info_init (&info, root, parse_subtree);
 
@@ -3618,6 +3640,8 @@ write_dir (MarkupDir *dir,
   gboolean retval = FALSE;
   char *whitespace;
 
+  dir->not_in_filesystem = TRUE;
+
   /* This dir will be deleted from the 
    * MarkupTree after syncing anyway ...
    */
@@ -3687,7 +3711,7 @@ save_tree (MarkupDir  *dir,
   new_fd = -1;
   f = NULL;
 
-  filename = markup_dir_build_path (dir, TRUE, save_as_subtree);
+  filename = markup_dir_build_file_path (dir, save_as_subtree);
   
   new_filename = g_strconcat (filename, ".new", NULL);
   new_fd = open (new_filename, O_WRONLY | O_CREAT, file_mode);
