@@ -123,7 +123,7 @@ entry_get_value(Entry* e, const gchar** locales, GError** err)
 
   gconf_log (GCL_DEBUG, "Cached schema value has locale \"%s\", looking for %s",
              sl ? sl : "null",
-             locales ? locales[0] : "null");
+             locales && locales [0] ? locales[0] : "null");
   
   /* optimize most common cases first */
   if (sl == NULL && (locales == NULL ||
